@@ -1,54 +1,9 @@
-;;;; $Id: mcl.lisp 687 2012-02-27 14:49:55Z ctian $
-;;;; $URL: svn://common-lisp.net/project/usocket/svn/usocket/tags/0.5.5/backend/mcl.lisp $
+;;;; $Id$
+;;;; $URL$
 
 ;; MCL backend for USOCKET 0.4.1
 ;; Terje Norderhaug <terje@in-progress.com>, January 1, 2009
 
-(in-package :ccl)
-
-(eval-when (:compile-toplevel :load-toplevel :execute)
-  (require :opentransport))
-
-;; MCL Issue 28: Passive TCP streams should be able to listen to the loopback interface
-;; see http://code.google.com/p/mcl/issues/detail?id=28 for details
-
-(defparameter *passive-interface-address* NIL
-  "Address to use for passive connections - optionally bind to loopback address while opening a tcp stream")
-
-(advise local-interface-ip-address
-  (or *passive-interface-address* (:do-it))
-  :when :around :name 'override-local-interface-ip-address)
-
-;; MCL Issue 29: Passive TCP connections on OS assigned ports
-;; see http://code.google.com/p/mcl/issues/detail?id=29 for details
-(advise ot-conn-tcp-passive-connect
-  (destructuring-bind (conn port &optional (allow-reuse t)) arglist
-    (declare (ignore allow-reuse))
-    (if (eql port #$kOTAnyInetAddress)
-	;; Avoids registering a proxy for port 0 but instead registers one for the true port:
-	(multiple-value-bind (proxy result)
-	    (let* ((*opentransport-class-proxies* NIL) ; makes ot-find-proxy return NIL
-		   (result (:do-it)) ;; pushes onto *opentransport-class-proxies*
-		   (proxy (prog1
-			      (pop *opentransport-class-proxies*)
-			    (assert (not *opentransport-class-proxies*))))
-		   (context (cdr proxy))
-		   (tmpconn (make-ot-conn :context context 
-					  :endpoint (pref context :ot-context.ref)))
-		   (localaddress (ot-conn-tcp-get-addresses tmpconn)))
-	      (declare (dynamic-extent tmpconn))
-	      ;; replace original set in body of function
-	      (setf (ot-conn-local-address conn) localaddress)
-	      (values
-	       (cons localaddress context)
-	       result))
-	  ;; need to be outside local binding of *opentransport-class-proxies* 
-	  (without-interrupts
-	      (push proxy *opentransport-class-proxies*))
-	  result)
-	(:do-it)))
-  :when :around :name 'ot-conn-tcp-passive-connect-any-address)
-
 (in-package :usocket)
 
 (defun handle-condition (condition &optional socket)
@@ -75,21 +30,26 @@
                             local-host local-port (protocol :stream))
   (when (eq nodelay :if-supported)
     (setf nodelay t))
-  (when (eq protocol :datagram)
-    (unsupported '(protocol :datagram) 'socket-connect))
-  (with-mapped-conditions ()
-    (let* ((socket
-            (make-instance 'active-socket
-              :remote-host (when host (host-to-hostname host)) 
-              :remote-port port
-              :local-host (when local-host (host-to-hostname local-host)) 
-              :local-port local-port
-              :deadline deadline
-              :nodelay nodelay
-              :connect-timeout (and timeout (round (* timeout 60)))
-              :element-type element-type))
-           (stream (socket-open-stream socket)))
-      (make-stream-socket :socket socket :stream stream))))
+  (ecase protocol
+    (:stream
+     (with-mapped-conditions ()
+       (let* ((socket
+               (make-instance 'active-socket
+		 :remote-host (when host (host-to-hostname host)) 
+                 :remote-port port
+                 :local-host (when local-host (host-to-hostname local-host)) 
+                 :local-port local-port
+                 :deadline deadline
+                 :nodelay nodelay
+                 :connect-timeout (and timeout (round (* timeout 60)))
+                 :element-type element-type))
+              (stream (socket-open-stream socket)))
+         (make-stream-socket :socket socket :stream stream))))
+    (:datagram
+     (with-mapped-conditions ()
+       (make-datagram-socket
+	 (ccl::open-udp-socket :local-address (and local-host (host-to-hbo local-host))
+			       :local-port local-port))))))
 
 (defun socket-listen (host port
                            &key reuseaddress
@@ -281,3 +241,27 @@
        #'ready-sockets
        (wait-list-waiters wait-list)))
     (nreverse result)))
+
+;;; datagram socket methods
+
+(defmethod initialize-instance :after ((usocket datagram-usocket) &key)
+  (with-slots (socket send-buffer recv-buffer) usocket
+    (setq send-buffer
+	  (ccl::make-TUnitData (ccl::ot-conn-endpoint socket)))
+    (setq recv-buffer
+	  (ccl::make-TUnitData (ccl::ot-conn-endpoint socket)))))
+
+(defmethod socket-send ((usocket datagram-usocket) buffer size &key host port (offset 0))
+  (with-mapped-conditions (usocket)
+    (with-slots (socket send-buffer) usocket
+      (unless (and host port)
+	(unsupported 'host 'socket-send))
+      (ccl::send-message socket send-buffer buffer size host port offset))))
+
+(defmethod socket-receive ((usocket datagram-usocket) buffer length &key)
+  (with-mapped-conditions (usocket)
+    (with-slots (socket recv-buffer) usocket
+      (ccl::receive-message socket recv-buffer buffer length))))
+
+(defmethod socket-close ((socket datagram-usocket))
+  nil) ; TODO
