@@ -336,7 +336,12 @@ happen. Use with care."
                            (backlog 5)
                            (element-type 'character))
   (let* ((reuseaddress (if reuse-address-supplied-p reuse-address reuseaddress))
-         (ip (host-to-vector-quad host))
+         (ip #+sbcl
+	     (if (and host
+		      (not (eq host *wildcard-host*)))
+		 (host-to-vector-quad host)
+		 (hbo-to-vector-quad sb-bsd-sockets-internal::inaddr-any))
+	     #-sbcl (host-to-vector-quad host))
          (sock (make-instance 'sb-bsd-sockets:inet-socket
                               :type :stream :protocol :tcp)))
     (handler-case
@@ -616,7 +621,7 @@ happen. Use with care."
       (raise-usock-err (sockint::wsa-get-last-error) socket)))
 
   (defun os-socket-handle (usocket)
-    (sockint::fd->handle (sb-bsd-sockets:socket-file-descriptor (socket usocket))))
+    (sb-bsd-sockets:socket-file-descriptor (socket usocket)))
 
   (defun bytes-available-for-read (socket)
     (sb-alien:with-alien ((int-ptr sb-alien:unsigned-long))
