@@ -1,5 +1,5 @@
-;;;; $Id: server.lisp 678 2011-11-11 01:40:53Z ctian $
-;;;; $URL: svn://common-lisp.net/project/usocket/svn/usocket/tags/0.5.5/server.lisp $
+;;;; $Id$
+;;;; $URL$
 
 (in-package :usocket)
 
@@ -76,12 +76,19 @@
   (declare (type stream stream))
   (terpri stream))
 
+(defun echo-tcp-handler (stream)
+  (loop
+     (when (listen stream)
+       (let ((line (read-line stream nil)))
+	 (write-line line stream)
+	 (force-output stream)))))
+
 (defun tcp-event-loop (socket function &optional arguments
                        &key element-type multi-threading)
   (let ((real-function #'(lambda (client-socket &rest arguments)
                            (unwind-protect
                                (multiple-value-bind (*remote-host* *remote-port*) (get-peer-name client-socket)
-				 (apply function (socket-stream client-socket) arguments))
+                                 (apply function (socket-stream client-socket) arguments))
                              (close (socket-stream client-socket))
                              (socket-close client-socket)
                              nil))))
